qx.Class.define("pf.base.Species",
{
    extend : qx.core.Object,
    
    members: {
        id : "",
        _m : 0.0,
        _q : 0,
        _n : 0.0,
        _t : 0.0,
        
        _update: function()
        {
        },
        
        setM: function(m)
        {
            m = parseFloat(m);
            if (m > 0.0) {
                this._m = m;
                this._update();
                return true;
            } else {
                return false;
            }
        },
        
        setQ: function(q)
        {
            this._q = parseInt(q);
            this._update();
        },
        
        setN: function(n)
        {
            n = parseFloat(n);
            if (n >= 0.0) {
                this._n = n;
                this._update();
                return true;
            } else {
                return false;
            }
        },
        
        setT: function(t)
        {
            t = parseFloat(t);
            if (t > 0.0) {
                this._t = t;
                this._update();
                return true;
            } else {
                return false;
            }
        },

        getN: function()
        {
            return this._n;
        },
        
        getQ: function()
        {
            return this._q;
        },
        
        getT: function()
        {
            return this._t;
        },
        
        getM: function()
        {
            return this._m;
        },
        
        getMassDensity: function()
        {
            return this._m*this._n;
        },
        
        getDebyeLength: function()
        {
            return Math.sqrt(this._t/(4*Math.PI*this._n))/Math.abs(this._q);
        },

        getPlasmaFrequencyW: function()
        {
            return Math.abs(this._q)*Math.sqrt(4*Math.PI*this._n/this._m);
        },
        getPlasmaFrequency: function()
        {
            return this.getPlasmaFrequencyW()/(2*Math.PI);
        },

        getInertialLength: function()
        {
            return pf.base.Bohr.c/this.getPlasmaFrequencyW();
        },

        getThermalVelocity: function()
        {
            return Math.sqrt(this._t/this._m);
        },
        
        getDeBroglieLength: function()
        {
            return pf.base.Thermo.thermalDeBroglieLength(this._m, this._t)/
                Math.sqrt(2*Math.PI);
        },
        
        getMinApproachDistance: function()
        {
            return this._q*this._q/this._t;
        },
        
        getTypicalDistance: function()
        {
            return Math.pow(4*Math.PI/3*this._n, -1/3);
        },
        
        getGyroFrequencyW: function(B)
        {
            return Math.abs(this._q)*B/this._m/pf.base.Bohr.c;
        },
        getGyroFrequency: function(B)
        {
            return this.getGyroFrequencyW(B)/(2*Math.PI);
        },
        
        getAlfvenVelocity: function(B)
        {
            if (B == 0) {
                return 0;
            } else {
                return B/Math.sqrt(4*Math.PI*this._m*this._n);
            }
        },
        
        getGyroRadius: function(B)
        {
            return this.getThermalVelocity()/this.getGyroFrequencyW(B);
        },
        
        getCoupling: function()
        {
            if (this._q == 0 || this._n == 0) {
                return 0;
            } else {
                var r = this.getTypicalDistance()/this.getDebyeLength();
                return 1/3*r*r;
            }
        },
        
        getHoltsmarkField: function()
        {
            return 2*Math.PI*Math.abs(this._q)*Math.pow(this._n*4/15, 2.0/3.0);
        },
        
        getClassicalRadius: function()
        {
            return pf.base.Bohr.alpha2*this._q*this._q/this._m;
        },
        
        getThomsonXsDiff: function()
        {
            var r0 = this.getClassicalRadius();
            return r0*r0;
        },
        
        getThomsonXsTotal: function()
        {
            return 8*Math.PI/3*this.getThomsonXsDiff();
        },
        
        getCyclotronLosses: function(B)
        {
            var w_c = this.getGyroFrequencyW(B);
            
            return 4/3*this._q*this._q*w_c*w_c*
                this._t/(this._m*Math.pow(pf.base.Bohr.c, 3))*this._n;
        },
        
        getPressure: function()
        {
            return this._n*this._t;
        },
        
        getChemicalPotential : function()
        {
            if (this.id == "e") {
                return pf.base.Thermo.muFD(this._n, this._t);
            } else {
                return pf.base.Thermo.muMB(this._m, this._n, this._t);
            }
        }
    },
        
    
    construct: function(id, m, q, n, t)
    {
        if (m <= 0.0 || n < 0.0 || t <= 0.0) {
            return null;
        }
         
        this.id = id;
        this._m  = m;
        this._q  = q;
        this._n  = n;
        this._t  = t;
        
        this._update();
    }
    
});
